@@ -39,7 +39,7 @@ static inline double powi(double base, int times)
 }
 #define INF HUGE_VAL
 #define TAU 1e-12
-#define Malloc(type,n) (type *)malloc((n)*sizeof(type))
+#define Malloc(type,n) (type *)malloc( (unsigned int) ((n)*sizeof(type)) )
 
 static void print_string_stdout(const char *s)
 {
